
local startFrame = CreateFrame("Frame", "startFrame")
startFrame:RegisterEvent("PLAYER_LOGIN")
startFrame:SetScript("OnEvent", function(self,event,...) 

 -- initialize vars from SavedVariables
 if not preferenceVars then
  preferenceVars = {
   menuFrameVisible = 1,
   menuTrayVisible = 1,
   verboseMode = 1,
   autofollowMode = 0,
   listenToName = UnitName("player"),
   listenToFullName = UnitName("player"),
  }
 end

 -- initialize main function
 main()

 -- initialize auto follow mode
 if preferenceVars.autofollowMode == 1 then
  changeTargetFrame:SetScript("OnEvent", targetThat)
 end -- else not necessary since the default is not set

 -- initialize menu frame
 if preferenceVars.menuFrameVisible == 0 then
  menuFrame:Hide()
 end -- else not necessary since the default is show()
 
 -- initialize tray icon
 if preferenceVars.menuTrayVisible == 0 then
  openMenuFrame:Hide()
 end -- else not necessary since the default is show()
 
 -- no need to initialize verbose mode
 
 -- initialize greeting
 print("[ l2target ] listening to: "..preferenceVars.listenToFullName)

 end)

function main()

 local addonVersion = GetAddOnMetadata("l2target", "version")
 local myName = UnitName("player")

 local timeSinceUpdateVar = 0
 local updateIntervalVar = 0.1
 local currentTargetVar = nil
 local nameplateVar = nil
 local visibleVar = FALSE

 local LibNameplate = LibStub("LibNameplate-1.0", true)

 local targetFrame = CreateFrame("Button","targetFrame",UIParent)
 targetFrame:SetNormalTexture("Interface\\TARGETINGFRAME\\UI-RaidTargetingIcon_8") -- Skull Raid Target Icon
 targetFrame:SetWidth(64)
 targetFrame:SetHeight(64)

 function targetFunction(self, elapsed) -- "Every 0.1 seconds, update the screen according to whether or not the nameplate in question is visible."
  timeSinceUpdateVar = timeSinceUpdateVar + elapsed
  if (timeSinceUpdateVar > updateIntervalVar) then
   if currentTargetVar then -- "Since frame numbering is reused in table, always re-lookup the nameplate frame value each time it goes visible."
    nameplateVar = LibNameplate:GetNameplateByName(currentTargetVar)
    if nameplateVar then -- "If it's not already visible, then do stuff.  Otherwise, nothing has changed and there's no point to make the computer do more work."
     if visibleVar == FALSE then
      visibleVar = TRUE
      targetFrame:SetPoint("CENTER", nameplateVar, "TOP", 0, 30)
      targetFrame:Show()
     end
    else -- "If it used to be visible, then do stuff.  Otherwise, nothing has changed and there's no point to make the computer do more work."
     if visibleVar == TRUE then
      visibleVar = FALSE
      targetFrame:ClearAllPoints()
      targetFrame:Hide()
     end
    end
   end
   timeSinceUpdateVar = 0;
  end
 end

 local targetEventFrame = CreateFrame("Frame", "targetEventFrame", UIParent) -- "Make a frame that exists just to handle the onUpdate events."
 targetEventFrame:SetScript("OnUpdate", targetFunction) -- "Set it to run the onUpdate function."

 SLASH_TARGETTHAT1 = "/targetthat"
 function targetThat()
  local targetName = UnitName("target")
  if targetName == nil then
   targetName = ""
  end
  if select(1, IsInInstance()) == 1 then -- this handles targeting while in a group and inside an instance
   SendAddonMessage("l2t", ">>>"..targetName, "INSTANCE_CHAT")
  else -- this handles targeting while in a group but not inside an instance
   if IsInGroup() then
    SendAddonMessage("l2t", ">>>"..targetName, "RAID")
   else -- this handles targeting while out of a group
    SendAddonMessage("l2t", ">>>"..targetName, "WHISPER", myName)
   end
  end
 end
 SlashCmdList["TARGETTHAT"] = targetThat;

 function chatParserFunction(self, event, arg1, arg2, arg3, arg4, ...)
  local validInput = string.find(arg2, ">>>")
  if validInput then
   if arg4 == preferenceVars.listenToName or arg4 == preferenceVars.listenToFullName then
    currentTargetVar = strsub(arg2, 4)
    if preferenceVars.verboseMode == 1 then
     if currentTargetVar ~= "" then
      print("[ l2target ] targeting: "..currentTargetVar)
     else 
      print("[ l2target ] target cleared")
     end
    end
   end
  end
 end

 local chatParserEvent = CreateFrame("Frame", "chatParserEvent")
 chatParserEvent:RegisterEvent("CHAT_MSG_ADDON")
 chatParserEvent:SetScript("OnEvent", chatParserFunction)

 SLASH_LISTENTO1 = '/listento';
 local function listenTo(msg, editbox)
  local targetName = UnitName("target");
  if targetName ~= nil then
   preferenceVars.listenToName = targetName
   local targetFullName = GetUnitName("target", true); 
   targetFullName = targetFullName:gsub("%s+", "")
   targetFullName = string.gsub(targetFullName, "%s+", "")
   preferenceVars.listenToFullName = targetFullName
   print("[ l2target ] listening to: "..targetFullName)
  end
 end
 SlashCmdList["LISTENTO"] = listenTo;

 SLASH_L2TARGET1 = '/l2target';
 local function toggleMenu(msg, editbox)
  if preferenceVars.menuFrameVisible == 1 then
   menuFrame:Hide()
   preferenceVars.menuFrameVisible = 0
  else
   menuFrame:Show()
   preferenceVars.menuFrameVisible = 1
  end
 end
 SlashCmdList["L2TARGET"] = toggleMenu;

 local openMenuFrame = CreateFrame("Button","openMenuFrame",Minimap)
 openMenuFrame:SetWidth(30)
 openMenuFrame:SetHeight(50)
 openMenuFrame:CreateTexture("TestTexture")
 TestTexture:SetAllPoints()
 TestTexture:SetTexture("Interface\\ACHIEVEMENTFRAME\\GuildTabard")
 TestTexture:SetTexCoord(0, .5, .5, 1)

 openMenuFrame:SetPoint("BOTTOMLEFT", "Minimap", "BOTTOMLEFT", 5, -23)
 openMenuFrame:SetFrameStrata("BACKGROUND")
 openMenuFrame:SetFrameLevel(1)
 Minimap:SetFrameLevel(3)
  openMenuFrame.text = openMenuFrame:CreateFontString(nil, "HIGH", "TextStatusBarText")
  openMenuFrame.text:SetPoint("CENTER", openMenuFrame, "CENTER", 3, -4)
  openMenuFrame.text:SetText("L2T") --openMenuFrame.text:SetTextColor(1,1,0)
 openMenuFrame:Show()
 openMenuFrame:SetScript("OnClick", toggleMenu)

 menuFrame = CreateFrame("Frame", "menuFrame", UIParent)
 menuFrame:SetFrameStrata("DIALOG")
 menuFrame:SetFrameLevel(0)
 menuFrame:ClearAllPoints()
 menuFrame:SetBackdrop(StaticPopup1:GetBackdrop())
 menuFrame:SetHeight(405)
 menuFrame:SetWidth(280)
 menuFrame:SetPoint("CENTER", 0, 0)
 menuFrame:SetMovable(true)
 menuFrame:EnableMouse(true)
 menuFrame:SetUserPlaced(true)
 menuFrame:SetScript("OnMouseDown", menuFrame.StartMoving)
 menuFrame:SetScript("OnMouseUp", menuFrame.StopMovingOrSizing)
  menuFrame.text = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrame.text:SetText("l2target v. "..addonVersion)
  menuFrame.text:SetPoint("CENTER", menuFrame, "TOP", 0, -40)
 menuFrame:Show()

 local menuFrameCloseButton = CreateFrame("Button", "menuFrameCloseButton", menuFrame, "UIPanelButtonTemplate")
 menuFrameCloseButton:SetSize(28 ,24) -- width, height
 menuFrameCloseButton:SetText(" X ")
 menuFrameCloseButton:SetPoint("TOPRIGHT", menuFrame, "TOPRIGHT", -10, -10)
 menuFrameCloseButton:SetScript("OnClick", toggleMenu)
 menuFrameCloseButton:Show()
 
 local menuFrameListenTo = CreateFrame("Button", "menuFrameListenTo", menuFrame, "UIPanelButtonTemplate")
 menuFrameListenTo:SetSize(170, 28)
 menuFrameListenTo:SetText(" Click here ")
 menuFrameListenTo:SetPoint("CENTER", menuFrame, "TOP", 5, -130)
 menuFrameListenTo:SetScript("OnClick", listenTo)
  menuFrameListenTo.text1 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameListenTo.text1:SetText("* If you want to receive targets: *\n\n1.  Target a player to listen to.")
  menuFrameListenTo.text1:SetPoint("CENTER", menuFrameListenTo, "LEFT", 76, 40)
  menuFrameListenTo.text2 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameListenTo.text2:SetText("2. 	 ")
  menuFrameListenTo.text2:SetPoint("CENTER", menuFrameListenTo, "LEFT", -10, 0)
 menuFrameListenTo:Show()

 local function listenTo2()
  preferenceVars.listenToName = myName
  preferenceVars.listenToFullName = myName
  print("[ l2target ] listening to: "..myName)
 end
 
 local menuFrameCall = CreateFrame("Button", "menuFrameCall", menuFrame, "UIPanelButtonTemplate")
 menuFrameCall:SetSize(170, 28)
 menuFrameCall:SetText(" Click here ")
 menuFrameCall:SetPoint("CENTER", menuFrame, "TOP", 5, -200)
 menuFrameCall:SetScript("OnClick", listenTo2)
  menuFrameCall.text1 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameCall.text1:SetText("* If you want to call targets: *")
  menuFrameCall.text1:SetPoint("CENTER", menuFrameCall, "LEFT", 76, 30)
  menuFrameCall.text2 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameCall.text2:SetText("1. 	 ")
  menuFrameCall.text2:SetPoint("CENTER", menuFrameCall, "LEFT", -10, 0)
  menuFrameCall.text3 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameCall.text3:SetText("2.  Create a macro with:       \n/targetthat                  ")
  menuFrameCall.text3:SetPoint("CENTER", menuFrameCall, "LEFT", 70, -35)
  menuFrameCall.text4 = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  menuFrameCall.text4:SetText("3.  Or, use auto follow mode:")
  menuFrameCall.text4:SetPoint("CENTER", menuFrameCall, "LEFT", 74, -65)
 menuFrameCall:Show()

 SLASH_AUTOFOLLOW1 = "/autofollow"
 local function autofollowModeFn()
  if preferenceVars.autofollowMode == 1 then
   changeTargetFrame:SetScript("OnEvent", nil)
   preferenceVars.autofollowMode = 0
   print("[ l2target ] auto follow mode off")
   autofollowButton:SetText("OFF")
  else 
   changeTargetFrame:SetScript("OnEvent", targetThat)
   preferenceVars.autofollowMode = 1
   print("[ l2target ] auto follow mode on")
   autofollowButton:SetText("ON")
  end
 end
 SlashCmdList["AUTOFOLLOW"] = autofollowModeFn;

 local autofollowButton = CreateFrame("Button", "autofollowButton", menuFrame, "UIPanelButtonTemplate")
 autofollowButton:SetSize(50, 28)
 if preferenceVars.autofollowMode == 1 then
  autofollowButton:SetText("ON")
 else 
  autofollowButton:SetText("OFF")
 end
 autofollowButton:SetPoint("CENTER", menuFrame, "TOP", 65, -290)
 autofollowButton:SetScript("OnClick", autofollowModeFn)
  autofollowButton.text = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  autofollowButton.text:SetText("Auto follow mode:")
  autofollowButton.text:SetPoint("CENTER", autofollowButton, "LEFT", -63, 0)
 autofollowButton:Show()

 SLASH_VERBOSEMODE1 = "/verbosemode"
 local function verboseModeFn()
  if preferenceVars.verboseMode == 1 then
   preferenceVars.verboseMode = 0
   print("[ l2target ] verbose mode off")
   verbosemodeButton:SetText("OFF")
  else 
   preferenceVars.verboseMode = 1
   print("[ l2target ] verbose mode on")
   verbosemodeButton:SetText("ON")
  end
 end
 SlashCmdList["VERBOSEMODE"] = verboseModeFn;

 local verbosemodeButton = CreateFrame("Button", "verbosemodeButton", menuFrame, "UIPanelButtonTemplate")
 verbosemodeButton:SetSize(50, 28)
 if preferenceVars.verboseMode == 1 then
  verbosemodeButton:SetText("ON")
 else 
  verbosemodeButton:SetText("OFF")
 end
 verbosemodeButton:SetPoint("CENTER", menuFrame, "TOP", 65, -320)
 verbosemodeButton:SetScript("OnClick", verboseModeFn)
  verbosemodeButton.text = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  verbosemodeButton.text:SetText("Verbose mode:")
  verbosemodeButton.text:SetPoint("CENTER", verbosemodeButton, "LEFT", -50, 0)
 verbosemodeButton:Show()

 function hideTray()
  if preferenceVars.menuTrayVisible == 1 then
   openMenuFrame:Hide()
   preferenceVars.menuTrayVisible = 0
   hideTrayButton:SetText("OFF")
  else
   openMenuFrame:Show()
   preferenceVars.menuTrayVisible =1 
   hideTrayButton:SetText("ON")
  end
 end

 local hideTrayButton = CreateFrame("Button", "hideTrayButton", menuFrame, "UIPanelButtonTemplate")
 hideTrayButton:SetSize(50, 28)
 if preferenceVars.menuTrayVisible == 1 then
  hideTrayButton:SetText("ON")
 else
  hideTrayButton:SetText("OFF")
 end
 hideTrayButton:SetPoint("CENTER", menuFrame, "TOP", 65, -350)
 hideTrayButton:SetScript("OnClick", hideTray)
  hideTrayButton.text = menuFrame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
  hideTrayButton.text:SetText("Minimap flag icon:")
  hideTrayButton.text:SetPoint("CENTER", hideTrayButton, "LEFT", -62, 0)
 hideTrayButton:Show()

 --====================--
 -- Rhusty's additions --
 --====================--

 addonPrefix = RegisterAddonMessagePrefix("l2t")
 local changeTargetFrame = CreateFrame("Frame", "changeTargetFrame")
 changeTargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED")

end
